<?php
/*
	mobiki - the wiki for the mobile world
	Author:  Martin Doering <martin@datenbrei.de>
	Project: http://mobiki.datenbrei.de
	License: http://mobiki.datenbrei.de/index.php?page=License
*/

	$ok = true;

	function check_version() {
		$version = current(explode("-", PHP_VERSION));
		if (version_compare($version, '5.5.0') >= 0) {
			$html = '<p style="color:green;">☑  — Your PHP version ' . $version . ' is ok!</p>' . "\n";
			$ok = false;
		}
		else {
			$html = '<p style="color:red;">☒  — You have PHP version ' . $version . ', which is not sufficient. You need at least version 5.5 because of support for "password_verify"!</p>' . "\n";
		}

		return $html;
		}
	
	function check_file($filename) {
		global $ok;
		if (file_exists($filename)) {
			if (is_writable($filename)) {
				$html = '<p style="color:green;">☑  —  "' . $filename . '" exists and is writable.</p>' . "\n";
			}
			else {
				$html = '<p style="color:red;">☒  —  "' . $filename . '" with permissions ' . decoct(fileperms($filename) & 0777) . ' exists, but is not writable by webserver user "' . get_current_user(). '"!</p>' . "\n";
				$ok = false;
			}
		}
		else {
			if (touch($filename))
				$html = '<p style="color:green;">☑  —  "' . $filename . '" does not exist, but permissions allow to create it later...</p>' . "\n";
			else { 
				$html = '<p style="color:red;">☒  —  "' . $filename . '" does not exist, and can not be created by webserver user "' . get_current_user(). '" because of wrong directory permissions!</p>' . "\n";
				$ok = false;
			}
			unlink($filename);
		}
		
		return $html;
	}
	
	function check_dir($usage, $filename) {
		global $ok;
		if (is_dir($filename)) {
			if (is_writable($filename)) {
				$html = '<p style="color:green;">☑  — The ' . $usage . ' Directory "' . realpath($filename) . '" exists and is writable.</p>' . "\n";
			}
			else {
				$html = '<p style="color:red;">☒  — The ' . $usage . ' Directory "' . realpath($filename) . '" with permissions ' . decoct(fileperms($filename) & 0777) . ' exists, but is not writable by webserver user "' . get_current_user(). '"!</p>' . "\n";
				$ok = false;
			}
		}
		else {
			$ok = false;
			if (mkdir($filename))
				$html = '<p style="color:red;">☒  — The ' . $usage . ' Directory "' . realpath($filename) . '" does not exist, but could be created by Blogless later...</p>' . "\n";
			else 
				$html = '<p style="color:red;">☒  — The ' . $usage . ' Directory "' . realpath($filename) . '" does not exist, and can not be created by webserver user "' . get_current_user(). '" because of wrong directory permissions!</p>' . "\n";
			unlink($filename);
		}
		
		return $html;
	}

	function check_sessions() {
		global $ok;
		if (session_start()) { 
			$html = '<p style="color:green;">☑  — The PHP session handler is correctly configured and operational.</p>' . "\n";
		}
		else {
			$ok = false;
			$html = '<p style="color:red;">☒  — The PHP session handler is not configured correctly! No session can be initiated! Blogless will not work!</p>' . "\n";
			if (ini_get('session.save_handler') != '') {
				$html .= '<p style="color:black;">☝  — The "session.save_handler" in your php.ini is set to "' . ini_get('session.save_handler') . '".</p>' . "\n";
				if (ini_get('session.save_path') != '')
					$html .= '<p style="color:black;">☝  — The "session.save_path" in your php.ini is set to "' . ini_get('session.save_path') . '".</p>' . "\n";
				else
					$html .= '<p style="color:black;">☝  — The "session.save_path" in your php.ini is not set at all - but should be set!</p>' . "\n";
			}
			else
				$html .= '<p style="color:black;">☝  — The "session.save_handler" in your php.ini is not set at all - but should be set!</p>' . "\n";
		}
		return $html;
	}

	function check_encoding() {
		global $ok;
		if (mb_internal_encoding() == 'UTF-8') { 
			$html = '<p style="color:green;">☑  — The internal character encoding of your PHP installation is "UTF-8".</p>' . "\n";
		}
		else {
			$ok = false;
			$html = '<p style="color:red;">☒  — Your PHP installation is not configured to handle UTF-8 encoding correctly! This will not work!</p>' . "\n";

			if (ini_get('default_charset') != '') {
				$html .= '<p style="color:black;">ℹ️  — The "default_charset" parameter in your php.ini is actually set to "' . ini_get('default_charset') . '".</p>' . "\n";
			}
		}
		$webserver = php_sapi_name();
		if ($webserver == 'apache2handler') {
			$html .= '<p style="color:gray;">☐ — Optional: Your webserver is Apache, check your httpd.conf, if AddDefaultCharset is set to "UTF-8".</p>' . "\n";
		}
		elseif ($webserver == 'cli-server') {
			$html .= '<p style="color:green;">☑ — You are running PHP\'s internal webserver, so no webserver encoding adjustments are needed.</p>' . "\n";
		}
		else {
			$html .= '<p style="color:gray;">☐ — Optional: Your webserver is reported as ' . $webserver . '. See, if it is configured to handle UTF-8 encoding.</p>' . "\n";
		}
		return $html;
	}

	/* Display current internal character encoding */

	
	
	
	if ($_SERVER["REQUEST_METHOD"] == "GET") {
		
		$html = "<html> \n";
		$html .= "<head> \n";
		$html .= "<title>Check Installation</title> \n";
		$html .= "<meta charset=UTF-8> \n";
		$html .= '<meta name="viewport" content="width=device-width, initial-scale=1, user-scalable=no">' . "\n";
		$html .= '<link rel="stylesheet" href="template.css" type="text/css" media="all">' . "\n";
		$html .= "</head> \n";
		$html .= "<body> \n";
		$html .= "<article> \n";
		$html .= '<h1>Check PHP Setup for mobiki</h1>' . "\n";
		$html .= '<header>' . "\n";
		$html .= '</header>' . "\n";

		$html .= '<h2>Check PHP Setup</h2>' . "\n";
		$html .= check_version();
		$html .= check_sessions();
		$html .= check_encoding('check.php');
		$html .= '<h2>Check Directory Permissions</h2>' . "\n";
		$html .= check_dir('Installation', '.');
		$html .= check_dir('Pages', 'pages/');
		$html .= check_dir('Descriptions', 'descriptions/');
		$html .= check_dir('Images', 'images/');
		$html .= '<h2>Check File Permissions</h2>' . "\n";
		$html .= check_file('config.php');
		$html .= check_file('password.php');
		$html .= check_file('check.php');
		$html .= '<hr>' . "\n";
		
		if ($ok) {
			$html .= '<p><strong>Everything ok! Your setup is prepared to run mobiki.</strong></p>' . "\n";
			$html .= '<br>' . "\n";
			$html .= '<p><a id="save" href="index.php">Go on and use mobiki by clicking here!</a></p>' . "\n";
		}
		else {
			$html .= '<p>Your Installation will not run Blogless! Check and fix your setup and run this test again by clicking below!</p>' . "\n";
			$html .= '<br>' . "\n";
			$html .= '<p><a id="save" href="check.php">Check again!</a></p>' . "\n";
		}
		$html .= "</article> \n";
		$html .= "</body>";
		$html .= "</html>";
		header('Content-type: text/html; charset=utf-8');
		header('Cache-Control: no-cache, must-revalidate');
		die($html);
	}
?>
